@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import Optional
+from typing import Optional, Union
 from datetime import datetime
 
 import disruptive
@@ -33,7 +33,7 @@ class _EventData(dtoutputs.OutputBase):
 
     Attributes
     ----------
-    timestamp : datetime
+    timestamp : datetime, str, optional
         Timestamp of when the event was received by a Cloud Connector.
 
     """
@@ -42,8 +42,17 @@ class _EventData(dtoutputs.OutputBase):
         """
         Constructs the _EventData object by inheriting parent.
 
+        Parameters
+        ----------
+        data : dict
+            Event data dictionary.
+        event_type : str
+            Name of the event type.
+
         """
 
+        self.timestamp: Optional[datetime | str] = None
+
         # If timestamp is provided, verify type and set attribute.
         if 'updateTime' in data:
             # Raw should be iso8601 str format, while
@@ -70,7 +79,10 @@ class _EventData(dtoutputs.OutputBase):
         dtoutputs.OutputBase.__init__(self, data)
 
     @classmethod
-    def from_event_type(cls, data: dict, event_type: str):
+    def from_event_type(cls,
+                        data: dict,
+                        event_type: str,
+                        ) -> Optional[_EventType]:
         """
         Constructs the appropriate child class from the provided event type.
 
@@ -92,13 +104,15 @@ class _EventData(dtoutputs.OutputBase):
 
         # Return the initialized class instance.
         if key:
-            return child._from_raw(data[event_type])
+            child_instance = child._from_raw(data[event_type])
         else:
             # Special case for labelsChanged event.
-            return child._from_raw(data)
+            child_instance = child._from_raw(data)
+        return child_instance
 
     @staticmethod
-    def __child_map(event_type: str):
+    def __child_map(event_type: str,
+                    ) -> tuple[Optional[_EventType], Optional[bool]]:
         """
         Based on provided event type, returns the
         child class and supporting information.
@@ -111,7 +125,6 @@ class _EventData(dtoutputs.OutputBase):
         """
 
         # Initialize the correct object.
-        # if event_type in [t.value['api_name'] for t in EventTypes]:
         if event_type in _EVENTS_MAP._api_names:
             out = (
                 getattr(
@@ -125,7 +138,7 @@ class _EventData(dtoutputs.OutputBase):
         dtlog.warning('Skipping unknown event type {}.'.format(event_type))
         return None, None
 
-    def _celsius_to_fahrenheit(self, celsius: float):
+    def _celsius_to_fahrenheit(self, celsius: float) -> float:
         """
         Converts Celsius temperature value to Fahrenheit.
 
@@ -168,12 +181,12 @@ class Touch(_EventData):
         """
 
         # Set parameter attributes.
-        self.timestamp = timestamp
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'touch')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'timestamp={}'\
             ')'
@@ -184,7 +197,7 @@ class Touch(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> Touch:
         """
         Constructs a Touch object from API response data.
 
@@ -210,7 +223,7 @@ class Touch(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.timestamp is not None:
             data['updateTime'] = self.timestamp
@@ -251,14 +264,14 @@ class Temperature(_EventData):
         """
 
         # Set parameter attributes.
-        self.celsius = celsius
-        self.fahrenheit = self._celsius_to_fahrenheit(celsius)
-        self.timestamp = timestamp
+        self.celsius: float = celsius
+        self.fahrenheit: float = self._celsius_to_fahrenheit(celsius)
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'temperature')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'celsius={}, '\
             'timestamp={}'\
@@ -271,7 +284,7 @@ class Temperature(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> Temperature:
         """
         Constructs a Temperature object from API response data.
 
@@ -298,7 +311,7 @@ class Temperature(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.celsius is not None:
             data['value'] = self.celsius
@@ -339,13 +352,13 @@ class ObjectPresent(_EventData):
         """
 
         # Set parameter attributes.
-        self.state = state
-        self.timestamp = timestamp
+        self.state: str = state
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'objectPresent')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'state={}, '\
             'timestamp={}'\
@@ -358,7 +371,7 @@ class ObjectPresent(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> ObjectPresent:
         """
         Constructs an ObjectPresent object from API response data.
 
@@ -385,7 +398,7 @@ class ObjectPresent(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.state is not None:
             data['state'] = self.state
@@ -402,7 +415,7 @@ class Humidity(_EventData):
     ----------
     temperature : float
         Temperature value in Celsius.
-    humidity : int
+    humidity : float
         Relative humidity in percent.
     timestamp : datetime
         Timestamp of when the event was received by a Cloud Connector.
@@ -430,15 +443,15 @@ class Humidity(_EventData):
         """
 
         # Set parameter attributes.
-        self.celsius = celsius
-        self.fahrenheit = self._celsius_to_fahrenheit(celsius)
-        self.humidity = humidity
-        self.timestamp = timestamp
+        self.celsius: float = celsius
+        self.fahrenheit: float = self._celsius_to_fahrenheit(celsius)
+        self.humidity: float = humidity
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'humidity')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'celsius={}, '\
             'humidity={}, '\
@@ -453,7 +466,7 @@ class Humidity(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> Humidity:
         """
         Constructs a Humidity object from API response data.
 
@@ -481,7 +494,7 @@ class Humidity(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.celsius is not None:
             data['temperature'] = self.celsius
@@ -525,13 +538,13 @@ class ObjectPresentCount(_EventData):
         """
 
         # Set parameter attributes.
-        self.total = total
-        self.timestamp = timestamp
+        self.total: int = total
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'objectPresentCount')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'total={}, '\
             'timestamp={}'\
@@ -544,7 +557,7 @@ class ObjectPresentCount(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> ObjectPresentCount:
         """
         Constructs a ObjectPresentCount object from API response data.
 
@@ -571,7 +584,7 @@ class ObjectPresentCount(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         data['total'] = self.total
         if self.timestamp is not None:
@@ -612,13 +625,13 @@ class TouchCount(_EventData):
         """
 
         # Set parameter attributes.
-        self.total = total
-        self.timestamp = timestamp
+        self.total: int = total
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'touchCount')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'total={}, '\
             'timestamp={}'\
@@ -631,7 +644,7 @@ class TouchCount(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> TouchCount:
         """
         Constructs a TouchCount object from API response data.
 
@@ -658,7 +671,7 @@ class TouchCount(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         data['total'] = self.total
         if self.timestamp is not None:
@@ -697,13 +710,13 @@ class WaterPresent(_EventData):
         """
 
         # Set parameter attributes.
-        self.state = state
-        self.timestamp = timestamp
+        self.state: str = state
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'waterPresent')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'state={}, '\
             'timestamp={}'\
@@ -716,7 +729,7 @@ class WaterPresent(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> WaterPresent:
         """
         Constructs a WaterPresent object from API response data.
 
@@ -743,7 +756,7 @@ class WaterPresent(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         data['state'] = self.state
         if self.timestamp is not None:
@@ -793,11 +806,11 @@ class NetworkStatusCloudConnector(dtoutputs.OutputBase):
         dtoutputs.OutputBase.__init__(self, {})
 
         # Unpack attributes.
-        self.cloudconnector_id = cloudconnector_id
-        self.signal_strength = signal_strength
-        self.rssi = rssi
+        self.cloudconnector_id: str = cloudconnector_id
+        self.signal_strength: int = signal_strength
+        self.rssi: int = rssi
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'cloudconnector_id={}, '\
             'signal_strength={}, '\
@@ -812,7 +825,7 @@ class NetworkStatusCloudConnector(dtoutputs.OutputBase):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> NetworkStatusCloudConnector:
         """
         Constructs a NetworkStatusCloudConnector object from API response data.
 
@@ -894,16 +907,17 @@ class NetworkStatus(_EventData):
         """
 
         # Set attributes.
-        self.signal_strength = signal_strength
-        self.rssi = rssi
-        self.transmission_mode = transmission_mode
-        self.cloud_connectors = cloud_connectors
-        self.timestamp = timestamp
+        self.signal_strength: int = signal_strength
+        self.rssi: int = rssi
+        self.transmission_mode: str = transmission_mode
+        self.cloud_connectors: list[NetworkStatusCloudConnector] = \
+            cloud_connectors
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'networkStatus')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'signal_strength={}, '\
             'rssi={}, '\
@@ -922,7 +936,7 @@ class NetworkStatus(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> NetworkStatus:
         """
         Constructs a NetworkStatus object from API response data.
 
@@ -959,7 +973,7 @@ class NetworkStatus(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.signal_strength is not None:
             data['signalStrength'] = self.signal_strength
@@ -1013,13 +1027,13 @@ class BatteryStatus(_EventData):
         """
 
         # Inherit parent _EventData class init with repacked data dictionary.
-        self.percentage = percentage
-        self.timestamp = timestamp
+        self.percentage: int = percentage
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'batteryStatus')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'percentage={}, '\
             'timestamp={}'\
@@ -1032,7 +1046,7 @@ class BatteryStatus(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> BatteryStatus:
         """
         Constructs a BatteryStatus object from API response data.
 
@@ -1059,7 +1073,7 @@ class BatteryStatus(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.percentage is not None:
             data['percentage'] = self.percentage
@@ -1109,15 +1123,15 @@ class LabelsChanged(_EventData):
         """
 
         # Set parameter attributes.
-        self.added = added
-        self.modified = modified
-        self.removed = removed
-        self.timestamp = timestamp
+        self.added: dict = added
+        self.modified: dict = modified
+        self.removed: list[str] = removed
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'labelsChanged')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'added={}, '\
             'modified={}, '\
@@ -1134,7 +1148,7 @@ class LabelsChanged(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> LabelsChanged:
         """
         Constructs a LabelsChanged object from API response data.
 
@@ -1163,7 +1177,7 @@ class LabelsChanged(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.added is not None:
             data['added'] = self.added
@@ -1215,14 +1229,14 @@ class ConnectionStatus(_EventData):
         """
 
         # Set parameter attributes.
-        self.connection = connection
-        self.available = available
-        self.timestamp = timestamp
+        self.connection: str = connection
+        self.available: list[str] = available
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'connectionStatus')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'connection={}, '\
             'available={}, '\
@@ -1237,7 +1251,7 @@ class ConnectionStatus(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> ConnectionStatus:
         """
         Constructs a ConnectionStatus object from API response data.
 
@@ -1265,7 +1279,7 @@ class ConnectionStatus(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.connection is not None:
             data['connection'] = self.connection
@@ -1313,14 +1327,14 @@ class EthernetStatus(_EventData):
         """
 
         # Set parameter attributes.
-        self.mac_address = mac_address
-        self.ip_address = ip_address
-        self.timestamp = timestamp
+        self.mac_address: str = mac_address
+        self.ip_address: str = ip_address
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'ethernetStatus')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'mac_address={}, '\
             'ip_address={}, '\
@@ -1335,7 +1349,7 @@ class EthernetStatus(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> EthernetStatus:
         """
         Constructs a EthernetStatus object from API response data.
 
@@ -1363,7 +1377,7 @@ class EthernetStatus(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.mac_address is not None:
             data['macAddress'] = self.mac_address
@@ -1405,13 +1419,13 @@ class CellularStatus(_EventData):
         """
 
         # Set parameter attributes.
-        self.signal_strength = signal_strength
-        self.timestamp = timestamp
+        self.signal_strength: int = signal_strength
+        self.timestamp: Optional[datetime | str] = timestamp
 
         # Inherit parent _EventData class init with repacked data dictionary.
         _EventData.__init__(self, self.__repack(), 'cellularStatus')
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         string = '{}.{}('\
             'signal_strength={}, '\
             'timestamp={}'\
@@ -1424,7 +1438,7 @@ class CellularStatus(_EventData):
         )
 
     @classmethod
-    def _from_raw(cls, data: dict):
+    def _from_raw(cls, data: dict) -> CellularStatus:
         """
         Constructs a CellularStatus object from API response data.
 
@@ -1451,7 +1465,7 @@ class CellularStatus(_EventData):
 
         return obj
 
-    def __repack(self):
+    def __repack(self) -> dict:
         data: dict = dict()
         if self.signal_strength is not None:
             data['signalStrength'] = self.signal_strength
@@ -1484,10 +1498,10 @@ class Event(dtoutputs.OutputBase):
         dtoutputs.OutputBase.__init__(self, event)
 
         # Unpack attributes from dictionary.
-        self.event_id = event['eventId']
-        self.event_type = event['eventType']
-        self.device_id = event['targetName'].split('/')[-1]
-        self.project_id = event['targetName'].split('/')[1]
+        self.event_id: str = event['eventId']
+        self.event_type: str = event['eventType']
+        self.device_id: str = event['targetName'].split('/')[-1]
+        self.project_id: str = event['targetName'].split('/')[1]
 
         # Since labelsChanged is the only event that does not
         # contain an updateTime field in data, we provide the
@@ -1502,7 +1516,7 @@ class Event(dtoutputs.OutputBase):
         )
 
     @classmethod
-    def from_mixed_list(cls, events: list[dict]):
+    def from_mixed_list(cls, events: list[dict]) -> list[Event]:
         """
         Construct Event objects for each event in list.
 
@@ -1533,7 +1547,12 @@ class __EventsMap():
 
     class __TypeNames():
 
-        def __init__(self, api_name, attr_name, class_name, is_keyed):
+        def __init__(self,
+                     api_name: str,
+                     attr_name: str,
+                     class_name: str,
+                     is_keyed: bool
+                     ) -> None:
             self.api_name = api_name
             self.attr_name = attr_name
             self.class_name = class_name
@@ -1622,3 +1641,10 @@ class __EventsMap():
 
 
 _EVENTS_MAP = __EventsMap()
+
+_EventType = Union[
+    Touch, Temperature, ObjectPresent, Humidity, ObjectPresentCount,
+    TouchCount, WaterPresent, NetworkStatus, NetworkStatusCloudConnector,
+    BatteryStatus, LabelsChanged, ConnectionStatus, EthernetStatus,
+    CellularStatus,
+]
